package org.bleb.paperboy.gui.htmlview;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
import org.bleb.paperboy.util.*;

/**
 * A simple extension of the HTMLEditor kit that fires Enter/Exit
 * hyperlink events. Also sets up our own default style sheet.
 *
 * @version 0.10 (01-Mar-2000)
 **/
public class PatchedHTMLEditorKit extends HTMLEditorKit
{
    // -- Distinguish between the two mouse events
    //
    public static final int JUMP = 0;
    public static final int MOVE = 1;

    // -- We need our own stylesheet
    //
    private StyleSheet styles   = new StyleSheet();

    // -- Manage the links
    //
    LinkController myController = new LinkController();


    /**
     * Call the constructor for <code>HTMLEditorKit</code> and set up
     * our style sheet.
     **/
    public PatchedHTMLEditorKit()
    {
        super();

        GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fontfamily     = env.getAvailableFontFamilyNames();
        //for (int i=0; i<fontfamily.length; i++) {
        //    DEBUG.println("Font: " + i + "\t" + fontfamily[i]);
        //}

        // -- Read in our own style sheet
        FileInput.Line in = new FileInput.Line("default.css");
        try {
            in.open();
            styles.loadRules(in.getReader(), null);
            in.close();
        } catch (Exception e) {
            DEBUG.println("PatchedHTMLEditorKit: Unable to load default.css");
            styles = super.getStyleSheet();
        }
    }


    /**
     * Install our mouse motion listeners to ensure we get all the appropriate
     * events.
     *
     * @param c JEditorPane to listen in
     **/
    public void install(JEditorPane c)
    {
        c.addMouseListener(myController);
        c.addMouseMotionListener(myController);
    }


    /**
     * Return our style sheet - allowing customisation
     *
     * @return the stylesheet to use to display documents
     **/
    public StyleSheet getStyleSheet()
    {
        return styles;
    }


    /**
     * Handle the links
     **/
    public static class LinkController extends MouseInputAdapter 
      implements Serializable
    {
        URL    currentUrl = null;
        String currentPos = "";

        // -- Updated mouseClicked event
        //
        public void mouseClicked(MouseEvent e)
        {
            JEditorPane editor = (JEditorPane) e.getSource();

            if (! editor.isEditable()) {
                Point pt = new Point(e.getX(), e.getY());
                int pos = editor.viewToModel(pt);
                if (pos >= 0) activateLink(pos, editor, JUMP);
            }
            return;
        }


        // -- New mouse moved event
        //
        public void mouseMoved(MouseEvent e)
        {
            JEditorPane editor = (JEditorPane) e.getSource();

            if (! editor.isEditable()) {
                Point pt = new Point(e.getX(), e.getY());
                int pos = editor.viewToModel(pt);
                if (pos >= 0) activateLink(pos, editor, MOVE);
            }
            return;
        }


        // -- Handle enter/exit and active events. Do it all here to
        //    prevent having to duplicate the href location code.
        //
        protected void activateLink(int pos, JEditorPane jep, int type)
        {
            Document doc = jep.getDocument();
            if (! (doc instanceof HTMLDocument)) return;
            
            HTMLDocument   hdoc       = (HTMLDocument) doc;
            Element        e          = hdoc.getCharacterElement(pos);
            AttributeSet   a          = e.getAttributes();
            AttributeSet   anchor     = (AttributeSet) a.getAttribute(HTML.Tag.A);
            boolean        shouldExit = false;
            HyperlinkEvent linkEvent  = null;

            String href = (anchor != null) ? 
                (String) anchor.getAttribute(HTML.Attribute.HREF) :
                null;
            
            if (href != null) {
                URL u;
                try {
                    u = new URL(hdoc.getBase(), href);
                } catch (MalformedURLException m) {
                    u = null;
                    DEBUG.println("PHEK.activateLink: Failed for "+hdoc.getBase() +
                                  ", " + href);
                }
                
                if ( (type == MOVE) && (u != null) &&
                     (! currentPos.equals( u.toString() )) ) {
                    // -- We've moved onto a new link, update status
                    //
                    linkEvent = new HyperlinkEvent(jep, 
                                                   HyperlinkEvent.EventType.ENTERED,
                                                   u, href);
                    currentUrl = u;
                    currentPos = u.toString();
                    
                } else if ( (type == JUMP) && (u != null) ) {
                    // -- The current link has been activated, jump
                    //
                    linkEvent = new HyperlinkEvent(jep, 
                                                   HyperlinkEvent.EventType.ACTIVATED,
                                                   u, href);
                    shouldExit = true;
                    
                } else {
                    // -- Something else, ignore it
                    //
                    return;
                }

                // -- Safety checking
                //
                if (linkEvent == null) {
                    DEBUG.println("PHEK.activateLink: Assert failed for " + u);
                    return;
                } else {
                    jep.fireHyperlinkUpdate(linkEvent);
                }
                
            } else if (currentUrl != null) {
                shouldExit = true;
            }
            
            // -- If activated a link now mark it is exited or if exited link
            //    fire off the exited event
            //
            if (shouldExit) {
                linkEvent = new HyperlinkEvent(jep,
                                               HyperlinkEvent.EventType.EXITED,
                                               currentUrl, null);
                jep.fireHyperlinkUpdate(linkEvent);
                currentUrl = null;
                currentPos = "";
            }
            return;
        }
    }
}




