package org.bleb.paperboy.plugin;

import java.util.*;
import java.io.*;
import org.bleb.paperboy.web.*;
import org.bleb.paperboy.util.*;

/**
 * Methods for managing the site plugins. Also provides the
 * interface which all plugins should implement. In addition to
 * the below <code>abstract</code> methods, should be provided:
 * <ul>
 * <li><code>public Vector stories;</code>
 * </ul>
 * The public variables violate OO-principles however it simplifies
 * the code for new plugins as they do not require methods to
 * return the above variables.
 *
 * @version 0.01 (07-Jan-1999)
 * @see OptionConfig
 **/
public abstract class Plugin
{
    private static Hashtable list;  // List of plugins & options

    public Vector stories;          // Stories found by a plugin instance

    /**
     * Initialise the plugin system, load configuration information
     **/
    public static void init()
    {
        String dir     = System.getProperty("paperboy.lib");
        String data    = System.getProperty("paperboy.data");
        if (dir == null) return;
        dir += "/plugins";

        String files[] = new File(dir.replace('/',
                                              File.separatorChar)).list();
        list = new Hashtable();
        for (int i=0; i<files.length; i++) {
            if (files[i].startsWith("Plugin") &&
                files[i].endsWith(".class")) {
                String name = files[i].substring(6);
                name = files[i].substring(0, files[i].length()-6);

                // -- Try loading it from the user dir first
                //
                FileInput.setRoot(data);
                OptionConfig conf = OptionConfig.getFile(name + ".conf");

                // -- If that fails, try the plugin dir, if still null
                //    assume no configuration options
                if (conf == null) {
                    FileInput.setRoot(dir);
                    conf = OptionConfig.getFile(name + ".conf");

                    if (conf == null) conf = new OptionConfig();
                }
                list.put(name, conf);
            }
        }

        FileInput.setRoot(".");
        DEBUG.println("Plugin.init: Complete.");
        return;
    }

    
    /**
     * Fetch the list of plugin names which are available.
     *
     * @return Enumeration is a list of plugin class names
     **/
    public static Enumeration list()
    {
        return list.keys();
    }


    /**
     * Get configuration options for a particular plugin
     *
     * @param s Plugin name, eg. <var>BBC</var>
     * @return OptionConfig for that plugin
     **/
    public static OptionConfig getConfig(String s)
    {
        return (OptionConfig)list.get(s);
    }

    
    /**
     * Start fetching the relevant sections. A list of stories will
     * be built up and stored in <var>stories</var>.
     *
     * @param conf OptionConfig containing configuration data
     **/
    public abstract void start(OptionConfig conf) throws Exception;
    
    
    /**
     * Update a Story's content - this will only be called once
     * for a given page.
     *
     * @param s Story to update
     **/
    public abstract void fetch(Story s) throws Exception;
}
