package org.bleb.paperboy.rule;

import org.bleb.paperboy.util.*;
import org.bleb.paperboy.plugin.*;
import java.util.*;
import java.io.*;

/**
 * Represents the parse tree for the Rules system
 *
 * @version 1.00 (05-Jan-1999)
 **/
public abstract class Rule extends Object
{
    private static Hashtable rules;


    /**
     * Returns a representation of the rule as a string.
     *
     * @return String the text used to build up the rule
     **/
    public abstract String toString();
    
    
    /**
     * Returns true if this rule matches the given text.
     *
     * @param text String containing the text to search
     * @param c configuration of NEAR etc.
     * @return boolean - true if text and rule match
     **/
    public abstract boolean matches(String text, RuleConfig c);


    /**
     * Create a rule from a String. If not possible then
     * throws an exception with an error message.
     *
     * @param s Rule as a string, eg. <code>"paper NEAR boy"</code>
     * @throw ParseException if not a valid String
     **/
    public static Rule getRule(String s) throws ParseException
    {
        RuleParser p = new RuleParser( new StringReader(s+"\n\n") );
        Rule       r = null;
        try {
            r = p.getLine();
        } catch (ParseException e) {
            String msg = e.getMessage();
            int    pos = msg.indexOf(" at line ");
            throw new ParseException(msg.substring(0, pos));
        } catch (Exception e) {
            throw new ParseException("Unknown error - " + e.getMessage());
        }
        if (r == null) throw new ParseException("Invalid rule");

        return r;
    }


    /**
     * Initialise the rules database. This can be called <em>after</em>
     * <code>Cache.init()</code> and <strong>must</strong> be called
     * after <code>Plugin.init()</code>
     **/
    public static void init()
    {
        FileInput.Obj in  = new FileInput.Obj("rules.dat");
        rules             = new Hashtable();
        if (in.open()) {
            Hashtable tmp = (Hashtable)in.read();
            Enumeration e = Plugin.list();

            while (e.hasMoreElements()) {
                String   name = (String)e.nextElement();
                String[] rule = (String[])tmp.get(name);
                if (rule == null) continue;

                Rule[]   list = new Rule[rule.length];
                DEBUG.println("Rule.init: Found " + rule.length +
                              " rule(s) for " + name);
                for (int i=0; i<rule.length; i++) {
                    try {
                        list[i] = Rule.getRule(rule[i]);
                    } catch (ParseException pe) {
                        DEBUG.println("Rule.init: Failed '" + rule[i] +
                                      "' - " + pe.getMessage());
                        list[i] = null;
                    }
                }
                rules.put(name, list);
            }
            in.close();
        }
        DEBUG.println("Rule.init: Complete.");
        return;
    }


    /**
     * Get the array of Rules to use for a particular plugin.
     * Plugin name should be of the form "PluginFoo".
     *
     * @param n Name of plugin
     * @return Array of rules to match against
     **/
    public static Rule[] getList(String n)
    {
        return (Rule[])rules.get(n);
    }


    /**
     * Set the array of Rules for a plugin.
     *
     * @param n Name of plugin
     * @param r Array of rules
     **/
    public static void setList(String n, Rule[] r)
    {
        rules.put(n, r);
        return;
    }


    /**
     * Save the array back to the file
     **/
    public static void writeBack()
    {
        Hashtable   tmp = new Hashtable();
        Enumeration e   = rules.keys();
        while (e.hasMoreElements()) {
            String   key  = (String)e.nextElement();
            Rule[]   list = (Rule[])rules.get(key);
            String[] save = new String[list.length];
            for (int i=0; i<list.length; i++) {
                save[i] = list[i].toString();
            }
            tmp.put(key, save);
        }
        
        FileOutput.Obj out = new FileOutput.Obj("rules.dat");
        if (out.open()) {
            out.write(tmp);
            if (out.close()) DEBUG.println("Rule.writeBack: Complete.");
        }
        return;
    }
}


