package org.bleb.paperboy.rule;

import java.util.*;

/**
 * A split in the rule parse tree
 *
 * @version 1.04 (09-Mar-2000)
 **/
public class RuleJoin extends Rule
{
    public static final int AND  = 0;
    public static final int OR   = 1;
    public static final int NEAR = 2;
    public static final int ANOT = 3;
    
    private Rule    left;
    private Rule    right;
    private int     type;
    private boolean brackets;
    
    
    /**
     * Builds a new join in the tree.
     *
     * @param a Reference to the left hand side of the tree
     * @param t int containing the type (see constants)
     * @param b Reference to the right hand side of the tree
     * @param brckts boolean, true if rule should be surrounded
     *   by parentheses
     * @exception ParseException when trying to create something like
     *   <code>(...) NEAR (...)</code>. Only literals can be used
     **/
    public RuleJoin(Rule a, int t, Rule b, boolean brckts) throws ParseException
    {
        super();
        left     = a;
        type     = t;
        right    = b;
        brackets = brckts;
        
        if ((type == NEAR) && (!(a instanceof RuleNode) ||
                               !(b instanceof RuleNode))) {
            throw new ParseException("NEAR can only be used with literals");
        }
    }
    
    
    public String toString()
    {
        StringBuffer value  = new StringBuffer("");
        String middle = "";
        
        switch (type) {
            case AND:  middle = " AND ";     break;
            case OR:   middle = " OR ";      break;
            case NEAR: middle = " NEAR ";    break;
            case ANOT: middle = " AND NOT "; break;
        }
        
        if (brackets) { value.append("("); }
        value.append(left.toString()).append(middle).append(right.toString());
        if (brackets) { value.append(")"); }
        
        return value.toString();
    }
    
    
    public boolean matches(String text, RuleConfig c)
    {
        boolean value = false;

        if ((left == null) || (right == null)) return false;

        switch (type) {
            case AND:  value = left.matches(text, c) && right.matches(text, c); break;
            case OR:   value = left.matches(text, c) || right.matches(text, c); break;
            case NEAR: value = checkNear(text, c); break;
            case ANOT: value = left.matches(text, c) && !(right.matches(text, c)); break;
        }
        
        return value;
    }
    
    
    /**
     * Checks if NEAR can be matched here. Returns a boolean if the two
     * nodes can be matched within a given tolerance.
     *
     * @param text String containing text to check
     * @param c configuration information (eg. proximity of NEAR)
     * @return true if nodes within distance
     **/
    private boolean checkNear(String text, RuleConfig c)
    {
        RuleNode a         = (RuleNode)left;
        RuleNode b         = (RuleNode)right;
        int      tolerance = c.getNearProximity();
        boolean  match     = false;
        int      aIndex    = 0;
        int      distance;
        
        Integer aPos[] = (Integer[])a.positions(text, c).toArray(new Integer[0]);
        Integer bPos[] = (Integer[])b.positions(text, c).toArray(new Integer[0]);

        if ((aPos == null) || (bPos == null)) return false;
        while ((aIndex < aPos.length) &&
               !match && !(aPos[aIndex] == null)) {
            
            int bIndex = 0;
            while ((bIndex < bPos.length) &&
                   !match && !(bPos[bIndex] == null)) {
                distance = aPos[aIndex].intValue() - bPos[bIndex].intValue();
                if (Math.abs(distance) <= tolerance) { match = true; }
                
                bIndex++;
            }
            
            aIndex++;
        }
        
        return match;
    }
}
