package org.bleb.paperboy.util;

import java.io.*;
import java.util.*;
import org.bleb.paperboy.web.*;

/**
 * Methods for storing WebObjects on a local filesystem. Can check for
 * presence, ensure all referenced images are stored and a usage
 * count is also used.
 *
 * @version 0.02 (04-Feb-2000)
 **/
public class Cache extends Object
{
    private static Hashtable usage;

    /**
     * Initialise the cache. Load usage table and set root for
     * file operations etc.
     **/
    public static void init()
    {
        FileOutput.setRoot(System.getProperty("paperboy.data"));
        FileInput.setRoot( FileOutput.getRoot() );

        // -- Read the usage count table
        //
        usage = null;
        FileInput.Obj in = new FileInput.Obj("files.dat");
        if (in.open()) {
            usage = (Hashtable)in.read();
            in.close();
        } else {
            usage = new Hashtable();
            DEBUG.println("Cache.init: Unable to open 'files.dat' - assuming empty");
        }

        DEBUG.println("Cache.init: Complete.");
        return;
    }


    /**
     * Get usage data for a file. If the key wasn't found an
     * empty hashtable is returned.
     *
     * @param obj WebObject to fetch data for
     * @return Hashtable of data; usage counts etc.
     **/
    public static Hashtable getHash(WebObject obj)
    {
        Hashtable data = (Hashtable)usage.get(obj.getOriginalURL());
        return (data != null) ? data : new Hashtable();
    }


    /**
     * Set usage data for a file. The usage data is <em>not</em>
     * rewritten to disk. The following fields are also updated:
     * <ul><li>Local URL (<code>obj.getURL()</code>),
     *     <li>Last Modified Date (<code>obj.getLastModified()</code>)
     * </ul>
     *
     * @param obj WebObject to set data for
     * @param data Hashtable of data to store
     **/
    public static void setHash(WebObject obj, Hashtable data)
    {
        data.put("url", obj.getURL());
        data.put("date", new Long(obj.getLastModified()));
        usage.put(obj.getOriginalURL(), data);

        return;
    }


    /**
     * Write the usage table object to a serialised object
     * stream.
     **/
    public static void writeUsage()
    {
        FileOutput.Obj out = new FileOutput.Obj("files.dat");
        if (out.open()) {
            out.write(usage);
            if (! out.close()) {
                DEBUG.println("Cache.writeUsage: Couldn't close usage");
            }
        } else {
            DEBUG.println("Cache.writeUsage: Couldn't open usage for writing");
        }
        return;
    }


    /**
     * Store a WebObject in the Cache. The object's current URL is
     * updated to point to the position on disk relative to the
     * data directory. The usage count is <em>not</em> updated.
     *
     * @param obj WebObject to store
     * @return true if successfully written
     **/
    public static boolean store(WebObject obj)
    {
        boolean written = false;
        String  output  = obj.getData();

        // -- Find the local filename to use
        //
        String ext  = Codecs.getExtension( obj.getOriginalURL() );
        String file = "";

        if (obj instanceof Story) {
            Story s = (Story)obj;
            ext     = ".html";
            output  = "<html>\n<head><title>" + s.getHeadline() +
                "</title></head>\n\n<body>\n" +
                "<h1>" + s.getHeadline() + "</h1>\n" +
                "<h3>" + s.getSummary() + "</h3>\n\n" +
                output + "\n\n</body>\n</html>\n";
        } else {
            file = "images/";
        }
        file += Codecs.hash( obj.getOriginalURL() ) + ext;

        // -- Write it out
        //
        if (FileOutput.exists(file)) {
            DEBUG.println("Cache.store: '" + file + "' already exists, overwriting");
        }
        FileOutput.Line out = new FileOutput.Line(file);
        if ( out.open() ) {
            out.print(output);
            if ( out.close() ) {
                obj.setURL(file);
                written = true;
            }
        }
        return written;
    }


    /**
     * Determine if a URL is stored in the Cache.
     *
     * @param url Absolute URL to check
     * @return true if this URL is stored in the Cache.
     **/
    public static boolean stored(String url)
    {
        return (usage.get(url) != null);
    }


    /**
     * Get the original URL for a cached file
     *
     * @param file Local URL
     * @return Original URL for <var>file</var>
     **/
    public static String originalURL(String file)
    {
        Enumeration enum = usage.keys();
        String      orig = "";

        while (enum.hasMoreElements()) {
            String key = (String)enum.nextElement();
            String url = (String)((Hashtable)usage.get(key)).get("url");
            if (url.equals(file)) {
                orig = key;
                break;
            }
        }

        return orig;
    }


    /**
     * Find the local filename from the original URL
     *
     * @param url Original URL
     * @return Local filename for <var>url</var>
     **/
    public static String localURL(String url)
    {
        Enumeration enum = usage.keys();
        Hashtable   data = null;
        String      file = null;

        while (enum.hasMoreElements()) {
            String def = (String)enum.nextElement();
            if (def.equals(url)) {
                data = (Hashtable)usage.get(def);
                file = (String)data.get("url");
                break;
            }
        }
        return file;
    }


    /**
     * Determine if an object is already known about.
     *
     * @param obj WebObject to check
     * @return true if this object is already in the Cache.
     **/
    public static boolean stored(WebObject obj)
    {
        return (usage.get( obj.getOriginalURL() ) != null);
    }


    /**
     * List stored objects in the Cache. The <var>match</var>
     * parameter will limit the list to object whose local
     * URL contains <var>match</var>. This will be most useful
     * when <var>match</var> = <code>{ "html", "images/" }</code>
     *
     * @param match String to use for limiting
     * @return Vector of original URLs
     **/
    public static Vector list(String match)
    {
        Enumeration enum = usage.keys();
        Vector      list = new Vector();
        while (enum.hasMoreElements()) {
            String key = (String)enum.nextElement();
            String url = (String)((Hashtable)usage.get(key)).get("url");
            if ((url.indexOf(match) != -1) && !(list.contains(key)))
                list.add(key);
        }

        return list;
    }


    /**
     * Delete the file handling this object. Also, remove its line
     * from the usage table but do not rewrite. This will delete the
     * file even if it has a usage count which is non-zero. Checks
     * should be performed in <code>WebObject</code>.
     *
     * @param obj WebObject to delete
     * @return true if successfully deleted
     **/
    public static boolean delete(WebObject obj)
    {
        boolean deleted = false;

        // -- If deleted then update the usage table, this prevents
        //    it getting out of sync
        //
        File f = new File( new FileOutput(obj.getURL()).getFile() );
        if (f.delete()) {
            usage.remove( obj.getOriginalURL() );
            //writeUsage();
            deleted = true;
            DEBUG.println("Cache.delete: Deleted '" + f + "'");
        } else {
            DEBUG.println("Cache.delete: Couldn't delete '" + f + "'");
        }

        return deleted;
    }


    /**
     * Get a file from the Cache. The data isn't imported for
     * efficiency. When needed it can be found using
     * <code>getData()</code>.
     *
     * @param url String of original URL to fetch
     * @see #getData
     * @return WebObject of file
     **/
    public static WebObject get(String url)
    {
        WebObject   obj  = null;
        String      file = null;
        Enumeration enum = usage.keys();
        Hashtable   data = null;

        while (enum.hasMoreElements()) {
            String def = (String)enum.nextElement();
            if (def.equals(url)) {
                data = (Hashtable)usage.get(def);
                file = (String)data.get("url");
                break;
            }
        }

        // -- If file is still null then url isn't cached
        //
        if (file == null) {
            DEBUG.println("Cache.get: Don't know about '" + url + "'");
            return null;
        }

        // -- Determine type of file and create appropriate object,
        //    then update the instance variables
        //
        if (file.startsWith("images/")) {
            obj = new WebObject(url);
        } else {
            Story objReal = new Story(url);
            objReal.setHeadline( (String)data.get("headline") );
            objReal.setSummary( (String)data.get("summary") );
            objReal.setSection( (String)data.get("section") );
            objReal.setImages( (Vector)data.get("images") );
            objReal.setIsRead( (Boolean)data.get("read") );
            obj = objReal;
        }
        obj.setLastModified( ((Long)data.get("date")).longValue() );
        obj.setURL(file);

        // -- Don't read in the file as it probably isn't needed,
        //    however can do at a later date using getData
        return obj;
    }


    /**
     * Fetch an object's data if necessary. If
     * <code>obj.isAvailable()</code> is true then this just exits.
     *
     * @param obj WebObject to fetch
     **/
    public static void getData(WebObject obj)
    {
        if (obj.isAvailable()) return;

        FileInput.Line in = new FileInput.Line(obj.getURL());
        if (in.open()) {
            obj.setData( in.read() );
            in.close();
        } else {
            DEBUG.println("Cache.getData: Unable to open '"+obj.getURL()+"'");
        }

        return;
    }
}







