package org.bleb.paperboy.util;

import java.net.URL;
/**
 * Provides some simple character translation routines, eg.
 * Base64 encode, filename hashing...
 *
 * @version 0.84 (03-Mar-2000)
 **/
public class Codecs extends Object
                    implements Testable
{
    private static byte[]  Base64EncMap;
 
    /**
     * This is executed upon loading, and stores the complex
     * Base 64 <-> ASCII table
     **/
    static
    {
        // RFC-2045: Base64 Alphabet
        byte[] map = {
            (byte)'A', (byte)'B', (byte)'C', (byte)'D', (byte)'E', (byte)'F',
            (byte)'G', (byte)'H', (byte)'I', (byte)'J', (byte)'K', (byte)'L',
            (byte)'M', (byte)'N', (byte)'O', (byte)'P', (byte)'Q', (byte)'R',
            (byte)'S', (byte)'T', (byte)'U', (byte)'V', (byte)'W', (byte)'X',
            (byte)'Y', (byte)'Z',
            (byte)'a', (byte)'b', (byte)'c', (byte)'d', (byte)'e', (byte)'f',
            (byte)'g', (byte)'h', (byte)'i', (byte)'j', (byte)'k', (byte)'l',
            (byte)'m', (byte)'n', (byte)'o', (byte)'p', (byte)'q', (byte)'r',
            (byte)'s', (byte)'t', (byte)'u', (byte)'v', (byte)'w', (byte)'x',
            (byte)'y', (byte)'z',
            (byte)'0', (byte)'1', (byte)'2', (byte)'3', (byte)'4', (byte)'5',
            (byte)'6', (byte)'7', (byte)'8', (byte)'9', (byte)'+', (byte)'/' };
        Base64EncMap = map;
    }
    

    /**
     * This method encodes the given string using the base64-encoding
     * specified in RFC-2045 (Section 6.8). It's used for example in the
     * "Basic" authorisation scheme.
     *
     * @param  s String to encode
     * @return The Base64-encoded version of <var>s</var>
     **/
    public static String base64encode(String s)
    {
        if (s == null)  return  null;

        byte data[] = s.getBytes();
        return new String(base64encode(data));
    }


    /**
     * This method encodes the given byte array using the base64-encoding
     * specified in RFC-2045 (Section 6.8).
     *
     * @param  data The data array
     * @return The Base64-encoded <var>data</var>
     **/
    public static byte[] base64encode(byte[] data)
    {
        if (data == null) { return  null; }

        int sidx, didx;
        byte dest[] = new byte[((data.length+2)/3)*4];

        // -- 3-byte to 4-byte conversion + 0-63 to ascii printable conversion
        //
        for (sidx=0, didx=0; sidx < data.length-2; sidx += 3)
        {
            dest[didx++] = Base64EncMap[(data[sidx] >>> 2) & 077];
            dest[didx++] = Base64EncMap[(data[sidx+1] >>> 4) & 017 |
                                       (data[sidx] << 4) & 077];
            dest[didx++] = Base64EncMap[(data[sidx+2] >>> 6) & 003 |
                                       (data[sidx+1] << 2) & 077];
            dest[didx++] = Base64EncMap[data[sidx+2] & 077];
        }

        if (sidx < data.length) {
            dest[didx++] = Base64EncMap[(data[sidx] >>> 2) & 077];
            if (sidx < data.length-1) {
                dest[didx++] = Base64EncMap[(data[sidx+1] >>> 4) & 017 |
                                           (data[sidx] << 4) & 077];
                dest[didx++] = Base64EncMap[(data[sidx+1] << 2) & 077];
            } else {
                dest[didx++] = Base64EncMap[(data[sidx] << 4) & 077];
            }
        }

        // -- Add padding as necessary
        //
        while (didx < dest.length) dest[didx++] = (byte) '=';
        return dest;
    }
    
    
    /**
     * Get a hashcode suitable for using as a filename from a URL.
     *
     * @param url String containing URL to encode
     * @return String consisting of a almost-unique key
     **/
    public static String hash(String url)
    {
        int  MULTIPLIER = 31;
        long NHASH      = 1073741824;
        long hash       = 0;
        char p[]        = url.toCharArray();
        
        for (int i=0; i<p.length; i++) {
            hash = MULTIPLIER * hash + (int)p[i];
        }
        
        int key = (int)(hash % NHASH);
        String value = base64encode(new Integer(key).toString());
        return value.replace('/', ',').replace('=', '_');
    }
    

    /**
     * Return the file extension of a URL/filename
     *
     * @param s String containing URL/filename
     * @return String containing short extension (incl. dot prefix)
     **/
    public static String getExtension(String u)
    {
        int pos = u.lastIndexOf('.');

        if (pos == -1) { return ""; }
        return u.substring(pos);
    }
        

    /**
     * Derive the canonical URL from a base and new URL.
     *
     * @param base String containing base URL for canonical
     * @param url String of new URL to append to base
     * @return Derived URL
     **/
    public static String deriveURL(String base, String url)
    {
        String canon;

        try {
            canon = (new URL( new URL(base), url )).toString();
        } catch (Exception e) {
            // -- Couldn't derive the URL, make a guess...
            canon = url;
        }
        return canon;
    }


    // -------------------------------------------------------------------
    // TEST METHOD
    // -------------------------------------------------------------------
    public boolean test()
    {
        String  test   = "http://test/wibble,foo=bar";
        boolean status = true;
        
        System.err.println("    (using \"" + test + "\")");
        
        try {
            String result;
            result = this.base64encode(test);
            result = this.hash(test);
            result = this.hash(result);
            result = this.base64encode(result);
            if (! this.getExtension("moo.html").equals(".html")) {
                System.err.println("|   getExtension(\"moo.html\") != \".html\"");
                status = false;
            }
        } catch (Exception e) {
            System.err.println("|   Exception: " + e.getMessage());
            status = false;
        }
        
        return status;
    }
}

