package org.bleb.paperboy.util;

import java.io.*;

/**
 * Simplify reading input from files.
 *
 * @see FileOutput
 * @version 0.01 (03-Feb-2000)
 **/
public class FileInput extends Object
{
    protected FileInputStream in;
    protected String          file;
    private static String root = "";

    /**
     * Basic constructor which should be followed with a call to
     * <code>setFile()</code> before any other methods are called.
     *
     * @see #setFile
     **/
    public FileInput()
    {
        super();
        in  = null;
        file = null;
    }

    /**
     * Constructor including file to use.
     *
     * @see #setFile
     **/
    public FileInput(String f)
    {
        super();
        in = null;
        setFile(f);
    }


    /**
     * Sets the root directory used for all file operations,
     * this can be used to simplify the filenames passed to
     * <code>setFile()</code>.
     *
     * @param dir String containing directory to use (UNIX-style)
     * @see #getRoot
     * @see FileOutput#setRoot
     **/
    public static void setRoot(String dir)
    {
        if (dir == null) {
            DEBUG.println("FileInput.setRoot: Being set to null!");
            return;
        }

        if (! dir.endsWith("/")) dir += "/";
        root = dir.replace('/', File.separatorChar);
        return;
    }
    
    
    /**
     * Returns the root directory currently being used, this is
     * in the local system-dependent format.
     *
     * @return String containing system-dependent offset directory
     * @see #setRoot
     **/
    public static String getRoot() { return root; }
    

    /**
     * Checks to see if the specified file in the offset directory
     * exists.
     *
     * @param file String containing UNIX-style file name to check
     * @return true if the file exists
     **/
    public static boolean exists(String file)
    {
        return FileOutput.exists(file);
    }
    
    
    /**
     * Set the file to be used for this object. This should be a
     * UNIX style path, eg. <code>dir/dir/file.ext</code> and it
     * will be converted into the appropriate system representation
     * here.
     *
     * @param f String containing UNIX-style file to use.
     **/
    public void setFile(String f)
    {
        file = root + f.replace('/', File.separatorChar);
        return;
    }


    /**
     * Returns the system-dependent filename being used by this
     * object.
     *
     * @return String containing filename on this system
     **/
    public String getFile() { return file; }


    /**
     * Open the file if possible. If not possible print an error
     * message and return false.
     *
     * @return true if the file was opened successfully.
     **/
    public boolean open()
    {
        if (file == null) return false;

        try {
            in = new FileInputStream(file);
        } catch (FileNotFoundException e) {
            DEBUG.println("FileInput.open: Unable to open file - " + e.getMessage());
            return false;
        }

        return true;
    }


    /**
     * Close the file if possible. If not possible print an error
     * message and return false.
     *
     * @return true if the file was closed successfully.
     **/
    public boolean close()
    {
        if (in == null) return false;

        try {
            in.close();
        } catch (IOException e) {
            DEBUG.println("FileInput.close: Unable to close file - " + e.getMessage());
            return false;
        }

        return true;
    }


    /**
     * Return true if at the end of the file.
     *
     * @return true if EOF
     **/
    public boolean eof()
    {
        if (in == null) return true;

        boolean eof = false;
        try {
            eof = ( in.available() == 0);
        } catch (IOException e) {
            eof = true;
        }
        return eof;
    }


    // -------------------------------------------------------------------
    // SUBCLASSES, PROVIDING SPECIFIC TYPES OF OUTPUT
    // -------------------------------------------------------------------

    /**
     * Subclass for reading text files line by line
     * @see java.io.BufferedReader
     **/
    public static class Line extends FileInput
    {
        private BufferedReader read;

        public Line(String f) { super(f); }

        public boolean open()
        {
            if (! super.open()) { return false; }
            read = new BufferedReader(new InputStreamReader(super.in));

            return true;
        }

        public boolean close()
        {
            try {
                read.close();
            } catch (IOException e) {
                DEBUG.println("FileInput::Line.close: Can't close Reader - "+e.getMessage());
                return false;
            }
            return (super.close());
        }

        public String readln()
        {
            String val = null;
            try {
                val = read.readLine();
            } catch (Exception e) {
                DEBUG.println("FileInput::Line.readln: Couldn't read object: " + e.getMessage());
            }
            return val;
        }

        public String read()
        {
            StringBuffer buffer    = new StringBuffer();
            char[]       inputChar = new char[4096];
            int          len;

            try {
                while ((len = read.read(inputChar, 0, inputChar.length)) != -1) {
                    buffer.append(inputChar, 0, len);
                }
            } catch (Exception e) {
                DEBUG.println("FileInput::Line.read: Couldn't read input stream - " + e.getMessage());
            }
            return buffer.toString();
        }

        public BufferedReader getReader()
        {
            return read;
        }
    }


    /**
     * Subclass for reading objects by serialisation.
     *
     * @see java.io.Serializable
     **/
    public static class Obj extends FileInput
    {
        private ObjectInputStream istream;

        public boolean open()
        {
            if (! super.open()) { return false; }
            try {
                istream = new ObjectInputStream(super.in);
            } catch (IOException e) {
                DEBUG.println("FileInput::Obj.open: Can't open InputStream - "+e.getMessage());
                return false;
            }
            return true;
        }

        public boolean close()
        {
            try {
                istream.close();
            } catch (IOException e) {
                DEBUG.println("FileInput::Obj.close: Can't close InputStream - "+e.getMessage());
                return false;
            }
            return (super.close());
        }

        public Obj(String f) { super(f); }

        public Object read()
        {
            Object val = null;
            try {
                val = istream.readObject();
            } catch (Exception e) {
                DEBUG.println("FileInput::Obj.read: Couldn't read object: " + e.getMessage());
            }
            return val;
        }
    }
}
