package org.bleb.paperboy.util;

import java.io.*;

/**
 * Simplify writing to files
 *
 * @version 0.04 (03-Feb-2000)
 * @see FileInput
 **/
public class FileOutput extends Object
                        implements Testable
{
    protected FileOutputStream out;
    protected String           file;
    private static String root = "";

    /**
     * Basic constructor which should be followed with a call to
     * <code>setFile()</code> before any other methods are called.
     *
     * @see #setFile
     **/
    public FileOutput()
    {
        super();
        out  = null;
        file = null;
    }

    /**
     * Constructor including file to use.
     *
     * @see #setFile
     **/
    public FileOutput(String f)
    {
        super();
        out = null;
        setFile(f);
    }


    /**
     * Sets the root directory used for all file operations,
     * this can be used to simplify the filenames passed to
     * <code>setFile()</code>.
     *
     * @param dir String containing directory to use (UNIX-style)
     * @see #getRoot
     **/
    public static void setRoot(String dir)
    {
        if (dir == null) {
            DEBUG.println("FileOutput.setRoot: Being set to null!");
            return;
        }

        if (! dir.endsWith("/")) dir += "/";
        root = dir.replace('/', File.separatorChar);
        return;
    }
    
    
    /**
     * Returns the root directory currently being used, this is
     * in the local system-dependent format.
     *
     * @return String containing system-dependent offset directory
     * @see #setRoot
     **/
    public static String getRoot() { return root; }
    

    /**
     * Checks to see if the specified file in the offset directory
     * exists.
     *
     * @param file String containing UNIX-style file name to check
     * @return true if the file exists
     **/
    public static boolean exists(String file)
    {
        File f = new File(root + file.replace('/', File.separatorChar));
        return f.exists();
    }
    
    
    /**
     * Set the file to be used for this object. This should be a
     * UNIX style path, eg. <code>dir/dir/file.ext</code> and it
     * will be converted into the appropriate system representation
     * here.
     *
     * @param f String containing UNIX-style file to use.
     **/
    public void setFile(String f)
    {
        file = root + f.replace('/', File.separatorChar);
        return;
    }


    /**
     * Returns the system-dependent filename being used by this
     * object.
     *
     * @return String containing filename on this system
     **/
    public String getFile() { return file; }


    /**
     * Open the file if possible. If not possible print an error
     * message and return false.
     *
     * @return true if the file was opened successfully.
     **/
    public boolean open()
    {
        if (file == null) return false;

        try {
            out = new FileOutputStream(file);
        } catch (FileNotFoundException e) {
            DEBUG.println("FileOutput.open: Unable to open file - " + e.getMessage());
            return false;
        }

        return true;
    }


    /**
     * Close the file if possible. If not possible print an error
     * message and return false.
     *
     * @return true if the file was closed successfully.
     **/
    public boolean close()
    {
        if (out == null) return false;

        try {
            out.close();
        } catch (IOException e) {
            DEBUG.println("FileOutput.close: Unable to close file - " + e.getMessage());
            return false;
        }

        return true;
    }


    // -------------------------------------------------------------------
    // TEST METHOD
    // -------------------------------------------------------------------
    public boolean test()
    {
        boolean status = true;
        String  file   = "test.txt";

        DEBUG.setState(true);
        this.setRoot(System.getProperty("user.home"));
        System.err.println("    (using \"" + this.getRoot() + file + "\")");

        this.setFile(file);
        status &= this.open();
        status &= this.close();
        DEBUG.setState(false);

        if (! this.exists(file)) {
            System.err.println("|   exists(\"" + file + "\") returned false");
            status = false;
        }
        
        if (status) {
            File f = new File(this.getFile());
            if (! f.delete()) {
                System.err.println("|   Unable to delete \"" + this.getFile() + "\"");
                status = false;
            }
            
        } else {
            System.err.println("|   Skipping deletion on failure...");
        }

        return status;
    }


    // -------------------------------------------------------------------
    // SUBCLASSES, PROVIDING SPECIFIC TYPES OF OUTPUT
    // -------------------------------------------------------------------

    /**
     * Subclass for outputting files line by line.
     *
     * @see java.io.PrintWriter
     **/
    public static class Line extends FileOutput
    {
        private PrintWriter print;

        public Line(String f) { super(f); }

        public boolean open()
        {
            if (! super.open()) { return false; }
            print = new PrintWriter(super.out);

            return true;
        }


        public boolean close()
        {
           print.close();
           return (super.close());
        }


        public void print(String s)   { print.print(s); }
        public void println(String s) { print.println(s); }
    }


    /**
     * Subclass for outputting objects by serialisation.
     *
     * @see java.io.Serializable
     **/
    public static class Obj extends FileOutput
    {
        private ObjectOutputStream ostream;

        public boolean open()
        {
            if (! super.open()) { return false; }
            try {
                ostream = new ObjectOutputStream(super.out);
            } catch (IOException e) {
                DEBUG.println("FileOutput::Obj.open: Can't open OutputStream - "+e.getMessage());
                return false;
            }
            return true;
        }

        public boolean close()
        {
            try {
                ostream.flush();
                ostream.close();
            } catch (IOException e) {
                DEBUG.println("FileOutput::Obj.close: Can't close OutputStream - "+e.getMessage());
                return false;
            }
            return (super.close());
        }

        public Obj(String f) { super(f); }

        public void write(Object o)
        {
            try {
                ostream.writeObject(o);
            } catch (IOException e) {
                DEBUG.println("FileOutput::Obj.write: Couldn't write object - " + e.getMessage());
            }
        }
    }
}
