package org.bleb.paperboy.web;

import java.io.*;
import java.util.*;
import org.bleb.paperboy.util.Testable;

/**
 * Stores an HTML segment and has various methods for processing it which are
 * based around "tags". These are Strings consisting of one or more HTML tags.
 * The following are all valid tags:
 * <code>
 * <ul>
 * <li>"&lt;p&gt;"
 * <li>"&lt;/h2&gt;&lt;p&gt;"
 * <li>"&lt;p&gt;&lt;a "
 * <li></code>etc...
 * </ul>
 *
 * @version 0.30 (07-Jan-2000)
 **/
public class HTMLBlock extends    Object
                       implements Testable
{
    private String content;     // Block of original text (no CR/LFs tho')
    private String contentLc;   // Lower case version of above
    private int    currentPos;  // Current position within block

    
    /**
     * Initialises the block with the empty String
     **/
    public HTMLBlock()
    {
        super();
        new HTMLBlock("");
    }
    
    
    /**
     * Initialises the block with a specific String
     *
     * @param s String to initialise with
     **/
    public HTMLBlock(String s)
    {
        super();
        setString(s);
    }
    
    
    /**
     * Set the content of this block to a specific String,
     * and reset the current position pointer.
     *
     * @param s String to replace current block with
     **/
    public void setString(String s)
    {
            char c;
        
        // Convert carriage returns/linefeeds to testable whitespace...
        StringBuffer sb = new StringBuffer(s);
        for (int i = 0; i<sb.length(); i++) {
            c = sb.charAt(i);
            
            if ((c == '\n') || (c == '\r') ||
                               (c == '\t')) {
                sb.setCharAt(i, ' ');
            }
        }
        
        content   = sb.toString();
        contentLc = content.toLowerCase();
        do {
            resetPosition();
        } while (replace("  ", " "));
        return;
    }
    

    /**
     * Get the content of this block as a String.
     *
     * @return String of contents
     **/
    public String toString()
    {
        return content;
    }
    
    
    /**
     * Reset current position to start of block.
     **/
    public void resetPosition()
    {
        currentPos = 0;
    }
    
    
    /**
     * Determines if a specific String is contained within the block
     * from the current position onwards.
     *
     * @see #containsIgnoreCase
     * @param search String to search for
     * @return true if the String can be found
     **/
    public boolean contains(String search)
    {
        return (content.indexOf(search, currentPos) != -1);
    }
    
    
    /**
     * Determines if a specific String is contained within the block
     * from the current position onwards in any combination of upper
     * and lower case.
     *
     * @see #contains
     * @param search String to search for
     * @return true if the String can be found
     **/
    public boolean containsIgnoreCase(String search)
    {
        String s = search.toLowerCase();
        return (contentLc.indexOf(s, currentPos) != -1);
    }
    
    
    /**
     * Find the next occurence of a string and replace it. The current position
     * is the next character after the replaced section. A wildcard
     * of '*' can be used <em>once</em> in the search string to
     * match multiple characters.
     *
     * @see #contains
     * @param search String to search for
     * @param replace String to replace <var>search</var> with
     * @return true if a replacement was made, otherwise false
     **/
    public boolean replace(String search, String replace)
    {
        int wildcard = search.indexOf("*");
        String tail  = "";

        if (wildcard != -1) {
            tail   = search.substring(wildcard + 1);
            search = search.substring(0, wildcard);
        }

        int p  = content.indexOf(search, currentPos);
        int pt = p;
        if (p == -1) { return false; }

        if (wildcard != -1) {
            pt = content.indexOf(tail, p + search.length());
            if (pt == -1) { return false; }
        } else {
            pt = p;
            tail = search;
        }
        
        StringBuffer temp = new StringBuffer(content);
        temp.replace(p, pt + tail.length(), replace); // Java 1.2 method
        
        content    = temp.toString();
        contentLc  = content.toLowerCase();
        currentPos = p + replace.length();
        
        return true;
    }
        
    
    /**
     * Generate a list of all the occurences of an attribute of the given tag
     * from the current position onwards. The list will be returned as an
     * Enumeration which can be iterated over with the standard functions.
     * Only attributes quoted with a double quote, ie. <code>&quot;</code>
     * will be found.
     *
     * @param  tag    String containing the tag, eg. <code>a</code>
     * @param  attrib String containing the attribute, eg. <code>href</code>
     * @return Vector containing list of the attribute's values
     **/
    public Vector getTagAttributes(String tag, String attrib)
    {
        Vector list = new Vector();
        int    pos  = currentPos;
        int    start, end;
        
        tag    = "<" + tag.toLowerCase() + " ";
        attrib = attrib.toLowerCase() + "=";
        
        while ((pos = contentLc.indexOf(tag, pos)) != -1) {
            start = contentLc.indexOf(attrib, pos);
            end   = contentLc.indexOf(">", pos);
            if ((start != -1) || (end < start)) {
                start += attrib.length() + 1;
                end    = contentLc.indexOf('"', start);
                pos    = end;
                list.addElement( content.substring(start, end) );
            } else {
                pos = end;
            }
        }
        return list;
    }
    
    
    /**
     * Returns the first URL in the block from the current position
     * which contains the given String. This is case sensitive.
     *
     * @param toFind String which is contained in wanted URL
     * @return String of URL which matches <i>toFind</i>
     **/
    public String findURL(String toFind)
    {
        Enumeration enum    = getTagAttributes("a", "href").elements();
        String      current = null;
        
        while (enum.hasMoreElements()) {
            current = (String)enum.nextElement();
            if (current.indexOf(toFind) != -1) { return current; }
        }
        
        return null;  // Not found any by this point
    }
         
    
    /**
     * Crop the content to the current position forwards to the
     * end of the String. Afterwards the current position will
     * be 0 (at the start of the block).
     *
     * @see #removeEnd
     * @return String of the thrown away content
     **/
    public String removeStart()
    {
        String garbage = content.substring(0, currentPos);
        content        = content.substring(currentPos);
        contentLc      = contentLc.substring(currentPos);
        currentPos     = 0;
        return garbage;
    }
    
    
    /**
     * Crop the content from the start of the String to the current
     * position, ie. discard everything after the current position.
     * Afterwards the current position will be 0 (at the start of
     * the block).
     *
     * @see #removeStart
     * @return String of the thrown away content
     **/
    public String removeEnd()
    {
        String garbage = content.substring(currentPos);
        content        = content.substring(0, currentPos);
        contentLc      = contentLc.substring(0, currentPos);
        return garbage;
    }
    
    
    /**
     * Moves forward to the next instance of the specified tag and
     * returns a Hashtable of key/value pairs for its attributes.
     * The search for the tag is case insensitive and all the keys
     * are stored in lowercase. The values are stored as-is.
     *
     * @param start String specifying tag to read, eg. 
     * &quot;<code>a</code>&quot;
     * @return Hashtable containing attributes of that tag
     * @throws TagNotFoundException if <i>tag</i> cannot be found
     **/
    public Hashtable getNextTag(String start)
            throws TagNotFoundException
    {
        Hashtable keys = new Hashtable(20, (float)0.75); // 20 elements
        
        moveToIgnoreCase("<" + start + " ", false);
        char tag[] = content.substring(currentPos,
                        content.indexOf('>', currentPos)).toCharArray();
        
        int FINDING_KEY    = 1;    // State constants for the
        int GETTING_VALUE  = 2;    // FSM
        
        StringBuffer key   = new StringBuffer();
        StringBuffer value = new StringBuffer();
        int  state         = FINDING_KEY;
        char quote         = ' ';  // Character which will close attribute
        
        for (int i=0; i<tag.length; i++) {
            if (state == FINDING_KEY) {
                if (tag[i] == '=') {
                    if ((new String("\"'")).indexOf(tag[i+1]) == -1) {
                        // Lookahead shows no quotes so it'll be like
                        // a number, or equivalent
                        quote = ' ';
                        state = GETTING_VALUE;
                    } else {
                        // There's a quote character so skip it
                        quote = tag[++i];
                        state = GETTING_VALUE;
                    }
                } else {
                    if (tag[i] != ' ') key.append(tag[i]);
                }
                
            } else if (state == GETTING_VALUE) {
                if ((tag[i] == quote) && (tag[i-1] != '\\')) {
                    // Quoted attribute is ending (if not prefixed
                    // with backslash)
                    keys.put( key.toString().toLowerCase(), value.toString() );
                    key   = new StringBuffer();
                    value = new StringBuffer();
                    state = FINDING_KEY;
                } else {
                    value.append(tag[i]);
                }
            }
        }
        
        // If still grabbing a value at the close of the tag stick
        // it into the hashtable anyway
        if (state == GETTING_VALUE) {
            keys.put( key.toString().toLowerCase(), value.toString() );
        }

        currentPos += tag.length + 1;
        return keys;
    }
    

    /**
     * Returns the selection defined by a tag pair found from the current
     * position onwards. Can be either inclusive or exclusive
     * of the tags but is case insensitive. Afterwards the current
     * position will be either before or after (depending on
     * inclusivity) the end tag. If <i>open</i> does not contain an
     * HTML close bracket, ie. &quot;<code>&gt;</code>&quot; yet it begins
     * with an HTML open bracketm ie. &quot;<code>&lt;</code>&quot; and
     * the contents are wanted <em>excluding</em> the tags the
     * content will begin after the next close angle bracket after
     * the opening tag.
     *
     * @param open String containing tag marking start of block
     * @param close String containing tag marking end of block
     * @param inclusive boolean if true then include the tags themselves
     * @see #crop
     * @return String delimited by the block <i>open</i> &lt;-&gt; <i>close</i>
     * @throws TagNotFoundException if either tag cannot be found
     **/
    public String getTagsSelection(String open, String close, boolean inclusive)
            throws TagNotFoundException
    {
        moveToIgnoreCase(open, inclusive);
        _skipTag(open, inclusive);
        
        int start = currentPos;
        moveToIgnoreCase(close, !inclusive);
        return content.substring(start, currentPos);
    }
    
    
    /**
     * Crop the block to the selection defined by two tags. As with
     * <code>getTagsSelection()</code> this is either inclusive or
     * exclusive and case-insensitive. The same rules for completing
     * the open tag apply. Afterwards the current position will be 0.
     *
     * @param open String containing tag marking start of block
     * @param close String containing tag marking end of block
     * @param inclusive boolean if true then include the tags themselves
     * @throws TagNotFoundException if either tag cannot be found
     * @see #getTagsSelection
     **/
    public void crop(String open, String close, boolean inclusive)
            throws TagNotFoundException
    {
        moveToIgnoreCase(open, inclusive);
        _skipTag(open, inclusive);
        removeStart();
        
        moveToIgnoreCase(close, !inclusive);
        removeEnd();
        resetPosition();
        return;
    }
    
    
    /**
     * Remove the block specified by two tags and returns it as a
     * String. As with <code>getTagsSelection()</code> this is either
     * inclusive or exclusive, and like <code>crop()</code> is
     * case-insensitive. Afterwards the current position is at the end
     * of the removed section.
     *
     * @param open String containing tag marking start of block
     * @param close String containing tag marking end of block
     * @param inclusive boolean if true then include the tags themselves
     * @return String containing section removed from the block
     * @throws TagNotFoundException if either tag cannot be found
     * @see #getTagsSelection
     * @see #crop
     **/
    public String removeBlock(String open, String close, boolean inclusive)
            throws TagNotFoundException
    {
        moveToIgnoreCase(open, inclusive);
        _skipTag(open, inclusive);
        
        int start = currentPos;
        moveToIgnoreCase(close, !inclusive);
        
        StringBuffer temp = new StringBuffer(content);
        temp.replace(start, currentPos, "");
        
        String removed = content.substring(start, currentPos);
        content        = temp.toString();
        contentLc      = content.toLowerCase();
        
        return removed;
    }
        
        
    // -- Performs the common operation of moving to an opening tag ------
    //
    private void _skipTag(String open, boolean inclusive)
        throws TagNotFoundException
    {
        if (open.startsWith("<") &&
           (! inclusive) &&
           (open.indexOf('>') == -1)) {
            moveTo(">", false);
        }
        return;
    }
    
     
    /**
     * Move forward to start/end of a tag. The search is case
     * sensitive in this version.
     *
     * @see #moveToIgnoreCase
     * @param toFind String containing start of tag to move to.
     * @param start boolean is true if should move to start of the tag.
     * @throws TagNotFoundException if the tag cannot be found.
     **/
    public void moveTo(String toFind, boolean start)
            throws TagNotFoundException
    {
        _moveTo(content, toFind, start);
        return;
    }
    
    
    /**
     * Move forward to the start/end of a tag. The search is
     * case insensitive.
     *
     * @see #moveTo
     * @param toFind String containing the start of tag to move to.
     * @param start boolean is true if should move to start of the tag
     * @throws TagNotFoundException if the tag cannot be found.
     **/
    public void moveToIgnoreCase(String toFind, boolean start)
            throws TagNotFoundException
    {
        _moveTo(contentLc, toFind.toLowerCase(), start);
        return;
    }
    
     
    // --
    // The common code for the above two methods...
    // --
    private void _moveTo(String block, String toFind, boolean start)
            throws TagNotFoundException
    {
        int newPos;
        
        if ((newPos = block.indexOf(toFind, currentPos)) == -1) {
            throw new TagNotFoundException(toFind);
        }
        
        currentPos = newPos + (start ? 0 : toFind.length());
        return;
    }

     
    
    // -------------------------------------------------------------------
    // TEST METHOD
    // -------------------------------------------------------------------
    public boolean test()
    {
        boolean status = true;
        
        String htmlsrc = "This block of <A\n HREF=\"http://www.w3.org/\">" +
                         "HTML</a> is to test the <i>HTMLBlock</i> class\n" +
                         "a part of <i>Paperboy</i>. The 3rd year project\n"+
                         "for <em>me</em>. <a href=\"two.html\">Page 2</a>\n"+
                         "hasn't been written yet...";
        
        setString(htmlsrc);
        System.err.println();  // No general information for this class...
        
        if ( this.toString().indexOf('\n') != -1 ) {
            System.err.println("|   toString() not converting CR/LF/tab");
            status = false;
        }

        try {
            String h = (String)this.getNextTag("a").get("href");
            if (! h.equals("http://www.w3.org/") ) {
                System.err.println("|   getNextTag(\"a\").get(\"href\") is not \"http://www.w3.org/\" (returned \"" + h + "\")");
                status = false;
            }
        } catch (Exception e) {
            System.err.println("|   getNextTag(\"a\") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        this.resetPosition();
        
        try {
            this.moveToIgnoreCase("<a ", false);
        } catch (Exception e) {
            System.err.println("|   moveTo(\"<a \") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        
        this.resetPosition();
        try {
            String contents = this.getTagsSelection("<a ", "</a>", false);
            if (! contents.equals("HTML")) {
                System.err.println("|   getTagsSelection(\"<a \", \"</a>\") did not return \"HTML\"");
                status = false;
            }
        } catch (Exception e) {
            System.err.println("|   getTagsSelection(\"<a \", \"</a>\") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        
        try {
            String removed = this.removeBlock("<i>", "</i>", true);
            if (! removed.equals("<i>HTMLBlock</i>")) {
                System.err.println("|   remove(\"<i>\", \"</i>\") did not return \"<i>HTMLBlock</i>\"");
                status = false;
            }
        } catch (Exception e) {
            System.err.println("|   remove(\"<i>\", \"</i>\") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        
        if (! this.replace("3rd*ar", "final bit")) {
            System.err.println("|   replace(\"3rd\", \"final\") did not succeed");
            status = false;
        }
        
        Enumeration enum    = this.getTagAttributes("a", "href").elements();
        if (! enum.hasMoreElements()) {
            System.err.println("|   getTagAttributes(\"a\", \"href\") has no elements");
            status = false;
        } else {
            String next = (String)enum.nextElement();
            if (! next.equals("two.html")) {
                System.err.println("|   getTagAttribute's first element is not \"two.html\" (returned \"" + next + "\")");
                status = false;
            }
        
            if (enum.hasMoreElements()) {
                System.err.println("|   getTagAttribute has more than one element");
                status = false;
            }
        }
        
        String found = this.findURL("two");
        if ((found == null) || (! found.equals("two.html"))) {
            System.err.println("|   findURL(\"two\") did not return \"two.html\" (returned \"" + found + "\")");
            status = false;
        }
        
        try {
            this.moveTo("been", false);
        } catch (Exception e) {
            System.err.println("|   moveTo(\"been\") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        
        this.removeStart();
        if (! this.toString().equals(" written yet...")) {
            System.err.println("|   removeStart() did not remove upto and including \"been\"");
            status = false;
        }
            
        try {
            this.moveTo(" yet", true);
        } catch (Exception e) {
            System.err.println("|   moveTo(\" yet\") threw an exception:");
            e.printStackTrace();
            status = false;
        }
        
        String end = this.removeEnd();
        if (! end.equals(" yet...")) {
            System.err.println("|   removeEnd() did not return \" yet...\" (returned \""+end+"\")");
            status = false;
        }
        
        if (! this.toString().equals(" written")) {
            System.err.println("|   removeEnd() did not remove everything after \" yet\"");
            status = false;
        }
        
        return status;
    }
}
