package org.bleb.paperboy.web;

import java.util.*;
import org.bleb.paperboy.util.*;

/**
 * Container for downloaded, part-downloaded and pending stories - this is
 * an model for a story, allowing headline; summary and body text to be
 * read and updated independently.
 * <p>
 * The <var>get</var> and <var>set</var> methods are all fairly
 * self-explanatory and will return <code>null</code> if unset.
 * <p>
 * Calling <code>setBody()</code> will update <code>isAvailable()</code>
 * to return true.
 *
 * @version 0.14 (05-Mar-2000)
 **/
public class Story extends WebObject
{
    private String  headline;
    private String  summary;
    private String  section;
    private boolean read;
    private Vector  images;
    
    
    /**
     * Initialise an empty story, with no associated URL or content
     **/
    public Story() { super(null); initVars(); }
    
    
    /**
     * Initialise an empty story for the passed URL.
     *
     * @param u String containing a URL to be used for this story.
     **/
    public Story(String u) { super(u); initVars(); }

    
    // -- Do the work of the constructors --------------------------------
    //
    private void initVars()
    {
        headline = null;
        summary  = null;
        section  = null;
        images   = null;
        read     = false;
        
        return;
    }
    
     
    public Vector  getImages()   { return images; }
    public String  getHeadline() { return headline; }
    public String  getSummary()  { return summary; }
    public String  getSection()  { return section; }
    public boolean isRead()      { return read; }
    
    public void setImages(Vector i)   { images = i; }
    public void setHeadline(String h) { headline = h; }
    public void setSummary(String s)  { summary = s; }
    public void setSection(String s)  { section = s; }
    public void setIsRead(Boolean b) {
        read = (b == null) ? false : b.booleanValue();
    }

    /**
     * Set the Story's content. However, IMGs and HREFs
     * are automagically translated into absolute
     * versions.
     *
     * @param s String to use as page
     **/
    public void setData(String s)
    {
        HTMLBlock   page = new HTMLBlock(s);
        Enumeration enum = page.getTagAttributes("img", "src").elements();

        images = new Vector();
        while (enum.hasMoreElements()) {
            String src = (String)enum.nextElement();
            String abs = Codecs.deriveURL(super.localURL, src);

            if (! images.contains(abs)) images.add(abs);
            //DEBUG.print("Story.setData: Changing img " + src + 
            //            " to " + abs + "... ");
            page.resetPosition();
            while (page.replace("\""+src+"\"", "\""+abs+"\""));
            //DEBUG.println("complete.");
        }
        DEBUG.println("Story.setData: All img tags rewritten.");

        page.resetPosition();
        enum = page.getTagAttributes("a", "href").elements();
        while (enum.hasMoreElements()) {
            String src = (String)enum.nextElement();
            if (! src.startsWith("http://")) {
                String abs = Codecs.deriveURL(super.localURL, src);

                //DEBUG.print("Story.setData: Changing href " + src + 
                //            " to " + abs + "... ");
                page.resetPosition();
                while (page.replace("\""+src+"\"", "\""+abs+"\""));
                //DEBUG.println("complete.");
            }
        }
        DEBUG.println("Story.setData: All hrefs rewritten.");

        super.setData( page.toString() );
        return;
    }


    /**
     * Absolutely set the data for this object. Just sets the raw data
     * for the object. No translations are done on images etc.
     *
     * @param d Data to use
     **/
    public void setRaw(String d)
    {
        super.setData(d);
        return;
    }


    /**
     * Store this Story in the Cache. Also, update the usage table
     * to include headline/summary data. Any referenced images will
     * also be fetched if necessary.
     **/
    public void store()
    {
        // -- Ensure images are fetched
        //
        HTMLBlock   page = new HTMLBlock(super.data);
        Enumeration enum = images.elements();
        Vector      list = new Vector();
        while (enum.hasMoreElements()) {
            String    img = (String)enum.nextElement();
            WebObject obj = WebObject.get(img);
            obj.store();
            
            String url = obj.getURL();
            list.add(url);
            page.resetPosition();
            // DEBUG.println("Story.store: Changing "+img+" to "+url);
            while (page.replace("\""+img+"\"", "\""+url+"\""));
        }
        super.data = page.toString();
        images     = list;

        // -- Write data to disk and continue if successful
        //
        if (! Cache.store(this)) return;

        // -- Update the usage table and rewrite
        //
        DEBUG.println("Story.store: Writing usage for '"+localURL+"'");
        storeHash();
        return;
    }


    /**
     * Ensure the hash data for this Story is up to date and written
     * to disk.
     **/
    public void storeHash()
    {
        updateHash();
        Cache.writeUsage();
        return;
    }


    /**
     * Ensure the hash data for this Story is up to date.
     **/
    public void updateHash()
    {
        Hashtable data = Cache.getHash(this);
        data.put("headline", headline);
        data.put("summary", summary);
        data.put("section", section);
        data.put("images", images);
        data.put("read", new Boolean(read));
        Cache.setHash(this, data);
        return;
    }


    /**
     * Delete the Story. Decrement the usage count for all the
     * referenced images. Updates the cached hash, unlike
     * <code>WebObject.delete()</code>
     **/
    public void delete()
    {
        Enumeration enum = images.elements();
        while (enum.hasMoreElements())
            WebObject.get( Cache.originalURL((String)enum.nextElement()) ).delete();

        Cache.delete(this);
        Cache.writeUsage();
        return;
    }
}





