package org.bleb.paperboy.web;

import java.util.*;
import org.bleb.paperboy.util.*;

/**
 * Container for data downloaded from the Web
 *
 * @version 0.01 (31-Jan-2000)
 **/
public class WebObject extends Object
{
    protected String  originalURL;
    protected String  data;
    protected long    lastModified;
    protected String  localURL;
    protected boolean available;


    /**
     * Create a shell WebObject.
     **/
    public WebObject() { new WebObject(null); }


    /**
     * Create a new WebObject from the specified URL.
     *
     * @param u String containing URL of original data.
     **/
    public WebObject(String u)
    {
        super();
        originalURL  = u;
        localURL     = u;
        available    = false;
        lastModified = 0;  // 00:00 01-Jan-1970 (GMT)
    }


    /**
     * Set the current URL for the object.
     *
     * @param u String containing current URL.
     **/
    public void setURL(String u) { localURL = u; }


    /**
     * Get the current URL for this object.
     *
     * @return String containing current URL of the data
     **/
    public String getURL() { return localURL; }


    /**
     * Set the original URL for this object. This should not be used
     * to change the value after it has been initially set as that
     * defeats the point of it...
     *
     * @param u String containing original URL of the data
     **/
    public void setOriginalURL(String u) { originalURL = u; }


    /**
     * Get the original URL for this object.
     *
     * @return String containing original URL for this object.
     **/
    public String getOriginalURL() { return originalURL; }


    /**
     * Set the last modified date for this object.
     *
     * @param l Long containing number of seconds since Jan 1st, 1970 GMT
     **/
    public void setLastModified(long l) { lastModified = l; }


    /**
     * Get the last modified time for this object (from the
     * HTTP connection.
     *
     * @return long containing number of seconds since 00:00 01-Jan-1970
     **/
    public long getLastModified() { return lastModified; }


    /**
     * Is the data for this object available or not?
     *
     * @return true if <code>setData()</code> != <var>null</var>
     * @see #setData
     **/
    public boolean isAvailable() { return available; }
    

    /**
     * Modify the data for this object.
     *
     * @param d String containing the data to be stored.
     **/
    public void setData(String d) { data = d; available = true; }


    /**
     * Extract the data stored in this object.
     *
     * @return String containing the data.
     **/
    public String getData() { return data; }


    /**
     * Get a new WebObject from a URL. If the object has been
     * already cached then fetch from the local disk otherwise
     * fetch it from the web.
     *
     * @param url String of original URL to fetch
     * @return WebObject from that URL (null if failed)
     **/
    public static WebObject get(String url)
    {
        if (Cache.stored(url)) return Cache.get(url);

        return getFromWeb(url);
    }


    /**
     * Fetch an object from the web, regardless of whether or
     * not it is already cached.
     *
     * @param url String of URL to fetch
     * @return WebObject from that URL (null if failed)
     **/
    public static WebObject getFromWeb(String url)
    {
        WebObject obj  = new WebObject(url);

        obj.update();
        return obj;
    }


    /**
     * Update a WebObject's data from its currently
     * known location, ie. <code>getURL()</code>.
     **/
    public void update()
    {
        Webpage site = new Webpage(localURL);

        try {
            site.connect();
            data         = site.getContent();
            lastModified = site.getConnection().getLastModified();
            site.close();
        } catch (Exception e) {
            DEBUG.println("WebObject.update: Failed for '" + localURL +"'");
        }

        return;
    }


    /**
     * Store this object in the Cache. Increments the usage count
     * for this object. If the object is already stored only
     * the usage count is increased.
     **/
    public void store()
    {
        // -- Write data to disk and continue if successful
        //
        if (!Cache.stored(this) && !Cache.store(this)) return;

        Hashtable data  = Cache.getHash(this);
        Integer   usage = (Integer)data.get("usage");
        int       count = 1;  // Default value

        // -- Increment count and rewrite
        //
        if (usage != null) count = usage.intValue() + 1;
        data.put("usage", new Integer(count));
        Cache.setHash(this, data); 
        DEBUG.println("WebObject.store: Writing usage for '"+localURL+"'");
        Cache.writeUsage();

        return;
    }


    /**
     * Decrement the count for this object. If it reaches 0,
     * delete the actual file from the Cache. Does not update
     * the cache hash. Call <code>Cache.writeUsage()</code> for that.
     **/
    public void delete()
    {
        Hashtable data  = Cache.getHash(this);
        Integer   usage = (Integer)data.get("usage");
        int       count = 0;

        // -- If no record of this file then exit quietly, our
        //    work is done.
        //
        if (usage == null) {
            return;
        } else {
            count = usage.intValue() - 1;
        }

        // -- Delete the file or update the usage count
        //
        if (count <= 0) {
            Cache.delete(this);
        } else {
            data.put("usage", new Integer(count));
            Cache.setHash(this, data);
        }

        return;
    }
}









