package org.bleb.paperboy.web;

import org.bleb.paperboy.util.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Gets a URL (HTTP only atm), even through an authenticated proxy
 * and can return various information from it.
 *
 * @version 0.30 (07-Jan-2000)
 **/
public class Webpage extends    Object
                     implements Testable
{
    private static String proxyUser = "";
    private static String proxyPass = "";
    private static String proxyAuth = null;
    
    private String            uri;
    private String            content;
    private HttpURLConnection http;
    private String            cookie;
    private String            pageAuth;
    private boolean           followRedirects;
    
    
    /**
     * Sets the username and password to use for the proxy.
     *
     * @param user User name to use for authenticated proxy
     * @param pass Password to use for authenticated proxy
     **/
    public static void setProxyAuth(String user, String pass)
    {
        proxyUser = user;
        proxyPass = pass;
        proxyAuth = "Basic " + Codecs.base64encode(user + ":" + pass);
    }
    
    
    /**
     * Sets the information to use for the proxy.
     *
     * @param host Host to use for proxy ("" for no proxy)
     * @param port Port to use for proxy (ignored if host == "")
     **/
    public static void setProxy(String host, int port)
    {
        Boolean use = new Boolean(! host.equals(""));
        Integer prt = new Integer(use.booleanValue() ? port : 0);
        
        Properties p = System.getProperties();
        p.put("http.proxyHost", host);
        p.put("http.proxyPort", prt.toString());
        p.put("http.useProxy", use.toString());
        DEBUG.println("Webpage.setProxy: " + host + ":" + port);
    }
    
    
    /**
     * Allows a web page to be fetched from a remote host.
     **/
    public Webpage()
    {
        super();
        new Webpage(null);
    }


    /**
     * Allows a web page to be fetched from a remote host.
     *
     * @param url URL to fetch from (only <code>http://</code> is supported)
     **/
    public Webpage(String url)
    {
        super();
        uri             = url;
        cookie          = null;
        content         = "";
        pageAuth        = null;
        followRedirects = true;
    }
    

    /**
     * Sets the URL for the page. This should not be called after
     * the connect() method.
     *
     * @param url String containing <code>http://</code> URL
     * @see #connect
     **/
    public void setURL(String url)
    {
        uri = url;
        return;
    }
        
                
    /**
     * Sets the cookie(s) to send for this connection.
     *
     * @param key   String of the cookie name
     * @param value String containing this cookie's value.
     **/
    public void setCookie(String key, String value)
    {
        cookie = key + "=" + value;
        return;
    }
    
    
    /**
     * Sets the username/password to use for any authentication
     * if necessary.
     *
     * @param user String containing user name
     * @param pass String containing password
     **/
    public void setPageAuth(String user, String pass)
    {
        pageAuth = Codecs.base64encode(user + ":" + pass);
        return;
    }
    

    /**
     * Set/unset follow page redirects. The default is <em>follow</em>
     * redirects.
     *
     * @param follow boolean true if redirects will be followed.
     **/
    public void setFollowRedirects(boolean follow)
    {
        followRedirects = follow;
        return;
    }
    
    
    /**
     * Connect to a server with the previously specified options and
     * store the result.
     *
     * @throws ConnectionFailed if unable to connect
     **/
    public void connect() throws ConnectionFailed
    {
        DEBUG.print("Webpage.connect: http://..." + uri.substring(uri.length() / 2) + "... ");
        HttpURLConnection.setFollowRedirects(followRedirects);

        try {
            URL page = new URL(uri);
            http = (HttpURLConnection)page.openConnection();
            if (proxyAuth != null) http.setRequestProperty("Proxy-Authorization",
                                                           proxyAuth);
            if (cookie    != null) http.setRequestProperty("Cookie", cookie);
            if (pageAuth  != null) http.setRequestProperty("Authorization",
                                                          "Basic " + pageAuth);
            http.connect();
            if (http.getResponseCode() >= 501) throw new ConnectionFailed(uri);

            // -- Read in the page
            //
            BufferedReader in = new BufferedReader(
                                        new InputStreamReader(http.getInputStream())
                                    );
        
            StringBuffer buffer    = new StringBuffer();
            char[]       inputChar = new char[4096];
            int          inputLen;

            while ((inputLen = in.read(inputChar,0,inputChar.length)) != -1) {
                    buffer.append(inputChar, 0, inputLen);
            }
            in.close();
        
            // -- Update our private variable
            //
            content = buffer.toString();
            DEBUG.println("complete");
            
        } catch (IOException e) {
            content = "<html>\n<head>\n<title>Error</title>\n</head>\n" +
                      "<body>\nAn IO error occured during the fetch: " +
                      e.getClass() + "(" + e.getMessage() + ")\n" +
                      "</body>\n</html>";
            DEBUG.println("failed - " + e.getClass() + " - " + e.getMessage());
        } catch (Exception e) {
            throw new ConnectionFailed(uri);
        }

        return;
    }
    
    
    /**
     * Returns the HttpURLConnection object for this connection.
     *
     * @returns The HttpURLConnection object
     **/
    public HttpURLConnection getConnection()
    {
        return http;
    }
    
    
    /**
     * Closes the connection opened by <code>connect()</code>.
     *
     * @see #connect
     **/
    public void close()
    {
        if (http != null) http.disconnect();
        return;
    }
    
    
    /**
     * Returns a string representation of the Webpage (the URL).
     *
     * @return The URL used to open the page.
     **/
    public String toString()
    {
        return uri;
    }
    
    
    /**
     * Returns the contents of the page.
     *
     * @return The contents of the page as a String.
     **/
    public String getContent()
    {
        return content;
    }


    // -------------------------------------------------------------------
    // TEST METHOD
    // -------------------------------------------------------------------
    public boolean test()
    {
        boolean status = true;
        
        this.setURL("http://flurble.themes.org/");
        // Webpage.setProxy("wwwcache.warwick.ac.uk", 3128);
        // Webpage.setProxyAuth("notused", "forjanet");
        
        System.err.println();
        long time;
        try {
            time = System.currentTimeMillis();
            this.connect();
            time = System.currentTimeMillis() - time;
            System.err.println("    (connect() took " + time + "ms)");
        } catch (Exception e) {
            System.err.println("|   connect() threw an exception:\n" +
                               "|       " + e.getMessage());
            status = false;
        }
        
        try {
            int rc = this.getConnection().getResponseCode();
            if (rc != HttpURLConnection.HTTP_OK) {
                System.err.println("|   getConnection().getResponseCode() = " + rc);
                System.err.println("|       getConnection().getResponseMessage() = \"" +
                                   this.getConnection().getResponseMessage() + "\")");
                status = false;
            }
        } catch (Exception e) {
            System.err.println("|    getConnection().getResponseCode() threw an exception:");
            System.err.println("|        " + e.getMessage());
            status = false;
        }
        
        if (status && (this.toString().indexOf("TestServlet") != -1)) {
            HTMLBlock test = new HTMLBlock(this.getContent());
            try { test.crop("parameters:", "</pre>", false); } catch(Exception e) {}
            while (test.replace("    ", "\n"));   // Globally split the lines
            System.out.println(test + "\n");
        }
        this.close();
        return status;
    }
}
